@@ -14,8 +14,7 @@
 #' packages your project depends on.
 #' \item \strong{Reproducible}: Packrat records the exact package versions you
 #' depend on, and ensures those exact versions are the ones that get installed
-#' wherever you go.
-#' }
+#' wherever you go.}
 #'
 #' Use \code{\link{init}} to create a new packrat project,
 #' \code{\link{snapshot}} to record changes to your project's library, and
@@ -70,8 +69,7 @@
 #'
 #' @docType package
 #' @name packrat
-#' @import utils
-#' @author Posit Software, PBC
+#' @author RStudio, Inc.
 NULL
 
 #' Initialize Packrat on a new or existing \R project
@@ -83,9 +81,7 @@ NULL
 #' \enumerate{
 #'
 #' \item Application dependencies are computed by examining the \R code
-#' throughout the project for \code{library} and \code{require} calls. You can
-#' opt out of this behavior by setting \code{infer.dependencies} to
-#' \code{FALSE}.
+#' throughout the project for \code{library} and \code{require} calls.
 #'
 #' \item A snapshot is taken of the version of each package currently used by
 #' the project as described in \code{\link{snapshot}}, and each package's
@@ -113,8 +109,6 @@ NULL
 #'   \code{\link{packrat-options}}.
 #' @param enter Boolean, enter packrat mode for this project after finishing a init?
 #' @param restart If \code{TRUE}, restart the R session after init.
-#' @param infer.dependencies If \code{TRUE}, infer package dependencies by
-#'   examining the \R code.
 #'
 #' @note
 #'
@@ -134,15 +128,24 @@ NULL
 init <- function(project = '.',
                  options = NULL,
                  enter = TRUE,
-                 restart = enter,
-                 infer.dependencies = TRUE)
-{
-  ## Get the initial directory structure, so we can rewind if necessary
-  project <- normalizePath(project, winslash = '/', mustWork = TRUE)
-  message("Initializing packrat project in directory:\n- ", surround(prettyDir(project), "\""))
+                 restart = enter) {
 
-  ## A set of files that packrat might generate as part of init -- we
-  ## enumerate them here to assist with later cleanup
+  opts <- default_opts()
+  if (!is.null(options)) {
+    for (i in seq_along(options)) {
+      name <- names(options)[[i]]
+      value <- options[[i]]
+      opts[[name]] <- options[[name]]
+    }
+  }
+
+  ## Get the initial directory structure, so we can rewind if necessary
+  project <- normalizePath(project,
+                           winslash = '/',
+                           mustWork = TRUE)
+  message("Initializing packrat project in directory:\n- ", surround(project, "\""))
+
+  ## A set of files that packrat might generate as part of init
   prFiles <- c(
     file.path(project, ".gitignore"),
     file.path(project, ".Rprofile"),
@@ -160,8 +163,68 @@ init <- function(project = '.',
   )
 
   withCallingHandlers(
-    initImpl(project, options, enter, restart, infer.dependencies),
+
+    expr = {
+
+      ## Force packrat mode off
+      if (isPackratModeOn())
+        off()
+
+      ## We always re-packify so that the current version of packrat present can
+      ## insert the appropriate auto-loaders
+      packify(project = project, quiet = TRUE)
+
+      descriptionFile <- file.path(project, 'DESCRIPTION')
+
+      if (file.exists(descriptionFile)) {
+        description <- as.data.frame(readDcf(descriptionFile))
+        package <- description$Package
+      }
+
+      ## Make sure the .Rprofile is up to date
+      augmentRprofile(project)
+      options <- initOptions(project, opts) ## writes out packrat.opts and returns generated list
+
+      # Take a snapshot
+      snapshotImpl(project,
+                   available.packages(contrib.url(activeRepos(project))),
+                   lib.loc = NULL,
+                   ignore.stale = TRUE,
+                   fallback.ok = TRUE)
+
+      # Use the lockfile to copy sources and install packages to the library
+      restore(project, overwrite.dirty = TRUE, restart = FALSE)
+
+      # Copy init.R so a user can 'start from zero' with a project
+      file.copy(
+        instInitFilePath(),
+        file.path(project, .packrat$packratFolderName, "init.R")
+      )
+
+      # Update project settings -- this also involves updating the .gitignore,
+      # etc
+      updateSettings(project, options)
+
+      ## Symlink system libraries always
+      symlinkSystemPackages(project = project)
+
+      message("Initialization complete!")
+
+      if (enter) {
+
+        setwd(project)
+
+        # Restart R if the environment is capable of it (otherwise enter packrat mode)
+        if (!restart || !attemptRestart())
+          on(project = project, clean.search.path = TRUE)
+      }
+
+      invisible()
+
+    }, ## expr
+
     error = function(e) {
+
       # Undo any changes to the directory that did not exist previously
       for (i in seq_along(priorStructure)) {
         file <- names(priorStructure)[[i]]
@@ -171,96 +234,13 @@ init <- function(project = '.',
           unlink(file, recursive = TRUE)
         }
       }
-    }
+
+    } ## error
+
   )
 
 }
 
-initImpl <- function(project = getwd(),
-                     options = NULL,
-                     enter = TRUE,
-                     restart = enter,
-                     infer.dependencies = TRUE)
-{
-  opts <- get_opts(project = project)
-  if (is.null(opts))
-    opts <- default_opts()
-
-  # Read custom Packrat options and apply them
-  customDefaultOptions <- getOption("packrat.default.project.options")
-  if (!is.null(customDefaultOptions)) {
-
-    # Validate the options (will stop on failure)
-    validateOptions(customDefaultOptions)
-
-    # Set the options
-    for (i in seq_along(customDefaultOptions)) {
-      name <- names(customDefaultOptions)[[i]]
-      opts[[name]] <- customDefaultOptions[[name]]
-    }
-  }
-
-  # NOTE: Explicitly set options should override default options set by
-  # the user
-  if (!is.null(options)) {
-    for (i in seq_along(options)) {
-      name <- names(options)[[i]]
-      opts[[name]] <- options[[name]]
-    }
-  }
-
-  # Force packrat mode off
-  if (isPackratModeOn())
-    off()
-
-  # We always re-packify so that the current version of packrat present can
-  # insert the appropriate auto-loaders
-  packify(project = project, quiet = TRUE)
-
-  # Make sure the .Rprofile is up to date
-  augmentRprofile(project)
-  options <- initOptions(project, opts) ## writes out packrat.opts and returns generated list
-
-  # If we don't yet have a lockfile, take a snapshot and then build the Packrat library.
-  if (!file.exists(lockFilePath(project = project))) {
-
-    snapshotImpl(project,
-                 lib.loc = NULL,
-                 ignore.stale = TRUE,
-                 fallback.ok = TRUE,
-                 infer.dependencies = infer.dependencies)
-
-    restore(project, overwrite.dirty = TRUE, restart = FALSE)
-
-  }
-
-  # Copy init.R so a user can 'start from zero' with a project
-  file.copy(
-    instInitFilePath(),
-    file.path(project, "packrat", "init.R")
-  )
-
-  # Update project settings -- this also involves updating the .gitignore,
-  # etc
-  updateSettings(project, options)
-
-  ## Symlink system libraries always
-  symlinkSystemPackages(project = project)
-
-  message("Initialization complete!")
-
-  if (enter) {
-
-    setwd(project)
-
-    # Restart R if the environment is capable of it (otherwise enter packrat mode)
-    if (!restart || !attemptRestart())
-      on(project = project, clean.search.path = FALSE)
-  }
-
-  invisible()
-}
-
 #' Apply the most recent snapshot to the library
 #'
 #' Applies the most recent snapshot to the project's private library.
@@ -270,12 +250,14 @@ initImpl <- function(project = getwd(),
 #'
 #' There are three common use cases for \code{restore}:
 #' \itemize{
-#' \item \strong{Hydrate}: Use \code{restore} after copying a project to a new machine
-#' to populate the library on that machine.
-#' \item \strong{Sync}: Use \code{restore} to apply library changes made by a
+#'   \item \strong{Hydrate}: Use \code{restore} after copying a project to a new
+#' machine to populate the library on that machine.
+#'
+#'   \item \strong{Sync}: Use \code{restore} to apply library changes made by a
 #' collaborator to your own library. (In general, you want to run \code{restore}
 #' whenever you pick up a change to \code{packrat.lock})
-#' \item \strong{Rollback}: Use \code{restore} to undo accidental changes made
+#'
+#'   \item \strong{Rollback}: Use \code{restore} to undo accidental changes made
 #' to the library since the last snapshot.
 #' }
 #'
@@ -283,64 +265,44 @@ initImpl <- function(project = getwd(),
 #' changes are necessary to currently loaded packages, you will need to restart
 #' \R to apply the changes (\code{restore} will let you know when this is
 #' necessary). It is recommended that you do this as soon as possible, because
-#' any library changes made between running \code{restore} and restarting \R
-#' will be lost.
+#' any library changes made between running \code{restore} and restarting \R will
+#' be lost.
 #'
-#' @note \code{restore} can be destructive; it will remove packages that were
-#'   not in the snapshot, and it will replace newer packages with older versions
-#'   if that's what the snapshot indicates. \code{restore} will warn you before
-#'   attempting to remove or downgrade a package (if \code{prompt} is
-#'   \code{TRUE}), but will always perform upgrades and new installations
-#'   without prompting.
+#' @note
+#' \code{restore} can be destructive; it will remove packages that were not in
+#' the snapshot, and it will replace newer packages with older versions if
+#' that's what the snapshot indicates. \code{restore} will warn you before
+#' attempting to remove or downgrade a package (if \code{prompt} is
+#' \code{TRUE}), but will always perform upgrades and new installations without
+#' prompting.
 #'
-#'   \code{restore} works only on the private package library created by
-#'   packrat; if you have other libraries on your path, they will be unaffected.
+#' \code{restore} works only on the private package library created by packrat;
+#' if you have other libraries on your path, they will be unaffected.
 #'
-#'   The \code{restart} parameter will only result in a restart of R when the R
-#'   environment packrat is running within makes available a restart function
-#'   via \code{getOption("restart")}.
+#' The \code{restart} parameter will only result in a restart of R when the
+#' R environment packrat is running within makes available a restart function
+#' via \code{getOption("restart")}.
 #'
-#'   To install packages hosted in private repositories on GitHub, GitLab, and
-#'   Bitbucket, you must either set the option
-#'   \code{packrat.authenticated.downloads.use.renv} to \code{TRUE} and ensure
-#'   that \code{curl} is available on your system, or ensure that the
-#'   \code{httr} package is available in your R library.
-#'
-#'   In addition, you must make credentials for your provider available in the
-#'   appropriate environment variable(s): \code{GITHUB_PAT}, \code{GITLAB_PAT},
-#'   and/or \code{BITBUCKET_USERNAME} and \code{BITBUCKET_PASSWORD}. These
-#'   environment variables are hidden from package installation subprocesses.
-#'
-#'   Packrat does not support installation from enterprise instances of GitHub,
-#'   GitLab, or Bitbucket.
-#'
-#'   Packrat selects a \code{tar} binary with the following heuristic: If a
-#'   \code{TAR} environment variable exists, Packrat will use that. Otherwise,
-#'   it will either look for a \code{tar} binary on the \code{PATH} on Unix, or
-#'   look for the system \code{tar} on Windows. If no binary is found in those
-#'   locations, it will use R's internal \code{tar} implementation, which may
-#'   cause errors with long filenames.
-#'
-#' @param project The project directory. When in packrat mode, if this is
-#'   \code{NULL}, then the directory associated with the current packrat project
-#'   is used. Otherwise, the project directory specified is used.
+#' @param project The project directory. When in packrat mode, if this is \code{NULL},
+#' then the directory associated with the current packrat project is used. Otherwise,
+#' the project directory specified is used.
 #' @param overwrite.dirty A dirty package is one that has been changed since the
-#'   last snapshot or restore. Packrat will leave these alone by default. If you
-#'   want to guarantee that \code{restore} will put you in the exact state
-#'   represented by the snapshot being applied, use \code{overwrite.dirty =
-#'   TRUE}.
+#' last snapshot or restore. Packrat will leave these alone by default. If you
+#' want to guarantee that \code{restore} will put you in the exact state
+#' represented by the snapshot being applied, use \code{overwrite.dirty = TRUE}.
 #' @param prompt \code{TRUE} to prompt before performing potentially destructive
-#'   changes (package removals or downgrades); \code{FALSE} to perform these
-#'   operations without confirmation.
+#' changes (package removals or downgrades); \code{FALSE} to perform these
+#' operations without confirmation.
 #' @param dry.run If \code{TRUE}, compute the changes to your packrat state that
 #'   would be made if a restore was performed, without actually executing them.
 #' @param restart If \code{TRUE}, restart the R session after restoring.
 #'
-#' @seealso \code{\link{snapshot}}, the command that creates the snapshots
-#'   applied with \code{restore}.
+#' @seealso
+#' \code{\link{snapshot}}, the command that creates the snapshots applied with
+#' \code{restore}.
 #'
-#'   \code{\link{status}} to view the differences between the most recent
-#'   snapshot and the library.
+#' \code{\link{status}} to view the differences between the most recent snapshot
+#' and the library.
 #'
 #' @export
 restore <- function(project = NULL,
@@ -350,27 +312,13 @@ restore <- function(project = NULL,
                     restart = !dry.run) {
 
   project <- getProjectDir(project)
-  stopIfNoLockfile(project)
+  stopIfNotPackified(project)
 
   if (!dry.run) {
     callHook(project, "restore", TRUE)
     on.exit(callHook(project, "restore", FALSE), add = TRUE)
   }
 
-  # Ensure the .libPaths() is set for the duration of this restore.
-  # Because it's possible for a user to attempt to restore a particular
-  # project while _not_ within packrat mode, we do not want the new
-  # .libPaths() to be persistent -- so we unset them at the conclusion
-  # of the restore. This is done to ensure downstream calls to e.g.
-  # `system.file()` are successful.
-  libDir <- libDir(project)
-  if (!file.exists(libDir(project)))
-    dir.create(libDir(project), recursive = TRUE)
-
-  oldLibPaths <- .libPaths()
-  .libPaths(c(libDir(project), oldLibPaths))
-  on.exit(.libPaths(oldLibPaths), add = TRUE)
-
   # RTools cp.exe (invoked during installation) can warn on Windows since we
   # use paths of the format c:/foo/bar and it prefers /cygwin/c/foo/bar.
   # Unfortunately, R's implementation of tar treats this warning output as
@@ -381,8 +329,6 @@ restore <- function(project = NULL,
     on.exit(Sys.setenv("CYGWIN" = cygwin), add = TRUE)
   }
 
-  # Validate the version of R used when restoring this project, and
-  # warn if the versions don't match.
   packages <- lockInfo(project)
   r_version <- lockInfo(project, 'r_version')
   if (!identical(as.character(getRversion()), r_version)) {
@@ -390,13 +336,19 @@ restore <- function(project = NULL,
             r_version)
   }
 
+  # Make sure the library directory exists
+  libDir <- libDir(project)
+  if (!file.exists(libDir)) {
+    dir.create(libDir, recursive=TRUE)
+  }
+
   # See if any of the packages that are currently in the library are dirty.
   # Dirty packages that are represented in the snapshot will be either ignored
   # (with a message) or overwritten, depending on the value of the
   # overwrite.dirty flag. Dirty packages that are not represented in the snapshot
   # (like git untracked) will be silently ignored in all cases.
 
-  libPkgNames <- rownames(installed.packages(libDir, noCache = TRUE))
+  libPkgNames <- rownames(installed.packages(libDir, noCache=TRUE))
   dirty <- !installedByPackrat(libPkgNames, libDir, TRUE)
   dirtyPackageNames <- libPkgNames[dirty]
 
@@ -415,17 +367,9 @@ restore <- function(project = NULL,
     pkgsToIgnore <- dirtyPackageNames[!dirtyPackageNames %in% pkgNames(packages)]
   }
 
-  # Configure repos globally to avoid explicitly passing the repos list to all
-  # downstream function calls.
-  repos <- lockInfo(project, 'repos')
-  externalRepos <- getOption('repos')
-  options(repos = repos)
-  on.exit({
-    options(repos = externalRepos)
-  }, add = TRUE)
-
-  # Install each package from CRAN or github/bitbucket/gitlab, from binaries when available and
+  # Install each package from CRAN or github, from binaries when available and
   # then from sources.
+  repos <- lockInfo(project, 'repos')
   restoreImpl(project, repos, packages, libDir,
               pkgsToIgnore = pkgsToIgnore, prompt = prompt,
               dry.run = dry.run,
@@ -467,6 +411,7 @@ clean <- function(packages = NULL,
                   force = FALSE) {
 
   project <- getProjectDir(project)
+  stopIfNotPackified(project)
 
   callHook(project, "clean", TRUE)
   on.exit(callHook(project, "clean", FALSE), add = TRUE)
@@ -541,8 +486,8 @@ unused_packages <- function(project = NULL,
   orphans <- setdiff(installedPkgNames,
                      packagesInUse)
 
-  ## Exclude 'manipulate', 'rstudio', and ignored packages
-  orphans <- setdiff(orphans, c("manipulate", "rstudio", opts$ignored.packages()))
+  ## Exclude 'manipulate', 'rstudio'
+  orphans <- setdiff(orphans, c("manipulate", "rstudio"))
   orphanRecs <- getPackageRecords(orphans,
                                   project = project,
                                   available = NULL,
@@ -589,19 +534,60 @@ packify <- function(project = NULL, quiet = FALSE) {
   }
 
   ## Copy over the packrat autoloader
-  augmentRprofile(project = project)
+  .Rprofile <- file.path(project, ".Rprofile")
+  autoloaderPath <- instInitRprofileFilePath()
+
+  if (!file.exists(.Rprofile)) {
+
+    file.copy(autoloaderPath, .Rprofile)
+
+  } else {
+
+    content <- readLines(.Rprofile)
+    autoloader <- readLines(autoloaderPath)
+
+    # if there is no content just overwrite the old file
+    if (!length(content) || identical(unique(content), "")) {
+      return(file.copy(autoloaderPath, .Rprofile, overwrite = TRUE))
+    }
+
+    # Remove the old autoloader
+    starts <- grep("#### -- Packrat Autoloader", content)
+    if (length(starts)) {
+      start <- min(starts)
+    } else {
+      start <- NULL
+    }
+
+    ends <- grep("### -- End Packrat Autoloader -- ####", content)
+    if (length(ends)) {
+      end <- max(ends)
+    } else {
+      end <- NULL
+    }
+
+    if (length(start) && length(end) && end > start)
+      content <- content[-c(start:end)]
+
+    if (length(content)) {
+      content <- c(content, "", autoloader)
+    } else {
+      content <- autoloader
+    }
+    cat(content, file = .Rprofile, sep = "\n")
+
+  }
 
   ## Copy in packrat/init.R
   file.copy(
     instInitFilePath(),
-    file.path(project, "packrat", "init.R"),
-    overwrite = TRUE
+    file.path(project, .packrat$packratFolderName, "init.R")
   )
 
   invisible()
 }
 
-lockInfo <- function(project, property = 'packages', fatal = TRUE) {
+lockInfo <- function(project, property='packages', fatal=TRUE) {
 
   project <- getProjectDir(project)
 
@@ -617,3 +603,4 @@ lockInfo <- function(project, property = 'packages', fatal = TRUE) {
   }
   readLockFile(lockFilePath)[[property]]
 }
+
