@@ -58,7 +58,7 @@ getPackageRecordsInstalledFromSource <- function(pkgs, lib.loc) {
       version = dcf$Version,
       source_path = dcf$InstallSourcePath,
       hash = hash(file.path(loc, "DESCRIPTION"))
-    ), class = c('packageRecord', 'source'))
+    ), class=c('packageRecord', 'source'))
   })
 }
 
@@ -78,13 +78,13 @@ getPackageRecordsLocalReposImpl <- function(pkg, repos, fatal = TRUE) {
   dcf <- as.data.frame(readDcf(file.path(path, "DESCRIPTION")), stringsAsFactors = FALSE)
   deps <- combineDcfFields(dcf, c("Depends", "Imports", "LinkingTo"))
   deps <- deps[deps != "R"]
-  structure(list(
+  record <- structure(list(
     name = pkg,
     source = 'source',
     version = dcf$Version,
     source_path = file.path(repoToUse, pkg),
     hash = hash(file.path(repoToUse, pkg, "DESCRIPTION"))
-  ), class = c('packageRecord', 'source'))
+  ), class=c('packageRecord', 'source'))
 }
 
 getPackageRecordsExternalSource <- function(pkgNames,
@@ -95,62 +95,35 @@ getPackageRecordsExternalSource <- function(pkgNames,
 
   lapply(pkgNames, function(pkgName) {
 
-    # The actual package record that will be populated by below logic.
-    result <- list()
+    pkgDescFile <- system.file('DESCRIPTION',
+                               package = pkgName,
+                               lib.loc = lib.loc)
 
-    # First, attempt to discover the actual installation for this package.
-    pkgDescFile <- system.file("DESCRIPTION", package = pkgName, lib.loc = lib.loc)
-    if (file.exists(pkgDescFile)) {
-
-      # If the package is currently installed, then we can return a package
-      # record constructed from the DESCRIPTION file.
+    # First, try inferring the package source from the DESCRIPTION file -- if it's unknown
+    # we might fall back to something in available.packages
+    if (nzchar(pkgDescFile)) {
       df <- as.data.frame(readDcf(pkgDescFile))
-      result <- suppressWarnings(inferPackageRecord(df, available))
-
-      # Normalize NULL source vs. 'unknown' source.
-      if (is.null(result$source))
-        result$source <- "unknown"
-
-      # If we don't know the package source, but the user has opted in
-      # to CRAN fallback, then warn the user and update the inferred source.
-      if (fallback.ok && result$source == "unknown") {
-        fmt <- paste(
-          "Package '%s %s' was installed from sources;",
-          "Packrat will assume this package is available from",
-          "a CRAN-like repository during future restores"
-        )
-        warning(sprintf(fmt, pkgName, result$version))
-        result$source <- "CRAN"
-      }
-
-    } else if (fallback.ok && pkgName %in% available[, "Package"]) {
-
-      # The package is not currently installed, but is available on CRAN.
-      # Snapshot the latest available version for this package from CRAN.
-      warning("Failed to infer source for package '", pkgName, "'; using ",
-              "latest available version on CRAN instead")
-
-      # Construct the package record by hand -- generate the minimal
-      # bits of the DESCRIPTION file, and infer the package record
-      # from that.
-      pkg <- available[pkgName,]
-      df <- data.frame(
-        Package = pkg[["Package"]],
-        Version = pkg[["Version"]],
-        Repository = "CRAN"
-      )
-      result <- suppressWarnings(inferPackageRecord(df, available))
-
-    } else {
-      # We were unable to determine an appropriate package record
-      # for this package; invoke the 'missing.package' callback.
-      return(missing.package(pkgName, lib.loc))
+      result <- suppressWarnings(inferPackageRecord(df))
     }
 
-    # Update the hash when available.
+    # If this failed, try falling back to something of the same name in 'available'
+    if (!nzchar(pkgDescFile) || (result$source == "unknown" && fallback.ok)) {
+      if (pkgName %in% rownames(available)) {
+        pkg <- available[pkgName,]
+        df <- data.frame(
+          Package = pkg[["Package"]],
+          Version = pkg[["Version"]],
+          Repository = "CRAN"
+        )
+      } else {
+        return(missing.package(pkgName, lib.loc))
+      }
+    }
+
+    # Now, we have either collected a package record from source, or an external repo
+    result <- inferPackageRecord(df)
     if (nzchar(pkgDescFile))
       result$hash <- hash(pkgDescFile)
-
     result
   })
 
@@ -167,48 +140,21 @@ getPackageRecordsLockfile <- function(pkgNames, project) {
   }
 }
 
-error_not_installed <- function(package, lib.loc) {
-  stop(
-    'The package "',
-    package,
-    '" is not installed in ',
-    ifelse(is.null(lib.loc), 'the current libpath', lib.loc)
-  )
-}
-
 # Returns a package records for the given packages
 getPackageRecords <- function(pkgNames,
                               project = NULL,
                               available = NULL,
                               recursive = TRUE,
                               lib.loc = NULL,
-                              missing.package = error_not_installed,
+                              missing.package = function(package, lib.loc) {
+                                stop('The package "', package, '" is not installed in ', ifelse(is.null(lib.loc), 'the current libpath', lib.loc))
+                              },
                               check.lockfile = FALSE,
-                              fallback.ok = FALSE,
-                              verbose = FALSE,
-                              .recursion.level = 1,
-                              .visited.packages = new.env(parent = emptyenv()))
-{
-  logger <- verboseLogger(verbose)
+                              fallback.ok = FALSE) {
+
   project <- getProjectDir(project)
   local.repos <- get_opts("local.repos", project = project)
 
-  # screen out empty package names that might have snuck in
-  pkgNames <- setdiff(pkgNames, "")
-
-  # Prior recursive steps may have already computed this package record and
-  # its recursive dependencies. Avoid constructing this package record.
-  priorPkgRecords <- dropNull(lapply(pkgNames, function(pkgName) {
-    if (exists(pkgName, envir = .visited.packages)) {
-      get(pkgName, envir = .visited.packages)
-    } else {
-      NULL
-    }
-  }))
-  if (length(priorPkgRecords)) {
-    pkgNames <- setdiff(pkgNames, sapply(priorPkgRecords, "[[", "name"))
-  }
-
   if (check.lockfile) {
     lockfilePkgRecords <- getPackageRecordsLockfile(pkgNames, project = project)
     pkgNames <- setdiff(pkgNames, sapply(lockfilePkgRecords, "[[", "name"))
@@ -264,7 +210,6 @@ getPackageRecords <- function(pkgNames,
 
   # Collect the records together
   allRecords <- c(
-    priorPkgRecords,
     lockfilePkgRecords,
     srcPkgRecords,
     manualSrcPkgRecords,
@@ -277,59 +222,46 @@ getPackageRecords <- function(pkgNames,
 
   # Now get recursive package dependencies if necessary
   if (recursive) {
-    .nnn <- length(allRecords)
-    .iii <- 0
     allRecords <- lapply(allRecords, function(record) {
-      .iii <<- .iii + 1
-      if (exists(record$name, envir = .visited.packages)) {
-        # We have already processed this package and computed its recursive
-        # dependencies. Avoid recursively computing its dependencies.
-        logger(sprintf("- (%3i / %3i; depth=%i) %s - using cached dependencies", .iii, .nnn, .recursion.level, record$name))
-        get(record$name, envir = .visited.packages)
-      } else {
-        # We have not already processed this package.
-        logger(sprintf("- (%3i / %3i; depth=%i) %s - calculating dependencies", .iii, .nnn, .recursion.level, record$name))
-        deps <- getPackageDependencies(pkgs = record$name,
-                                       lib.loc = lib.loc,
-                                       available.packages = available)
-        if (!is.null(deps)) {
-          record$depends <- getPackageRecords(
-            deps,
-            project = project,
-            available,
-            TRUE,
-            lib.loc = lib.loc,
-            missing.package = missing.package,
-            check.lockfile = check.lockfile,
-            fallback.ok = fallback.ok,
-            verbose = verbose,
-            .recursion.level = .recursion.level + 1,
-            .visited.packages = .visited.packages
-          )
-        }
-        .visited.packages[[record$name]] <- record
-        record
+      deps <- getPackageDependencies(pkgs = record$name,
+                                     lib.loc = lib.loc,
+                                     available.packages = available)
+      if (!is.null(deps)) {
+        record$depends <- getPackageRecords(
+          deps,
+          project = project,
+          available,
+          TRUE,
+          lib.loc = lib.loc,
+          missing.package = missing.package,
+          check.lockfile = check.lockfile,
+          fallback.ok = fallback.ok
+        )
       }
+
+      record
     })
   }
 
   allRecords
 }
 
-# Return TRUE when the data frame for this package has the given RemoteType.
-hasRemoteType <- function(df, remoteType) {
-  # Do not compare with 'identical'; RemoteType may be a factor.
-  return (!is.null(df$RemoteType) && df$RemoteType == remoteType)
-}
-
 # Reads a description file and attempts to infer where the package came from.
-# Currently works only for packages installed from CRAN or from GitHub/Bitbucket/Gitlab using
+# Currently works only for packages installed from CRAN or from GitHub using
 # devtools 1.4 or later.
-inferPackageRecord <- function(df, available = availablePackages()) {
+inferPackageRecord <- function(df) {
   name <- as.character(df$Package)
   ver <- as.character(df$Version)
 
-  if (length(df$GithubRepo) || hasRemoteType(df, "github")) {
+  if (!is.null(df$Repository) &&
+        identical(as.character(df$Repository), 'CRAN')) {
+    # It's CRAN!
+    return(structure(list(
+      name = name,
+      source = 'CRAN',
+      version = ver
+    ), class=c('packageRecord', 'CRAN')))
+  } else if (!is.null(df$GithubRepo)) {
     # It's GitHub!
     return(structure(c(list(
       name = name,
@@ -339,77 +271,25 @@ inferPackageRecord <- function(df, available = availablePackages()) {
       gh_username = as.character(df$GithubUsername),
       gh_ref = as.character(df$GithubRef),
       gh_sha1 = as.character(df$GithubSHA1)),
-      c(gh_subdir = as.character(df$GithubSubdir)),
-      c(remote_host = as.character(df$RemoteHost)),
-      c(remote_repo = as.character(df$RemoteRepo)),
-      c(remote_username = as.character(df$RemoteUsername)),
-      c(remote_ref = as.character(df$RemoteRef)),
-      c(remote_sha = as.character(df$RemoteSha))
-    ), class = c('packageRecord', 'github')))
-  } else if (hasRemoteType(df, "bitbucket")) {
-    # It's Bitbucket!
-    return(structure(c(list(
-      name = name,
-      source = 'bitbucket',
-      version = ver,
-      remote_repo = as.character(df$RemoteRepo),
-      remote_username = as.character(df$RemoteUsername),
-      remote_ref = as.character(df$RemoteRef),
-      remote_sha = as.character(df$RemoteSha)),
-      c(remote_host = as.character(df$RemoteHost)),
-      c(remote_subdir = as.character(df$RemoteSubdir))
-    ), class = c('packageRecord', 'bitbucket')))
-  } else if (hasRemoteType(df, "gitlab")) {
-    # It's GitLab!
-    return(structure(c(list(
-      name = name,
-      source = 'gitlab',
-      version = ver,
-      remote_repo = as.character(df$RemoteRepo),
-      remote_username = as.character(df$RemoteUsername),
-      remote_ref = as.character(df$RemoteRef),
-      remote_sha = as.character(df$RemoteSha)),
-      c(remote_host = as.character(df$RemoteHost)),
-      c(remote_subdir = as.character(df$RemoteSubdir))
-    ), class = c('packageRecord', 'gitlab')))
+      c(gh_subdir = as.character(df$GithubSubdir))
+    ), class=c('packageRecord', 'github')))
   } else if (identical(as.character(df$Priority), 'base')) {
     # It's a base package!
     return(NULL)
-  } else if (length(df$Repository) && identical(as.character(df$Repository), 'CRAN')) {
-    # It's CRAN!
-    return(structure(list(
-      name = name,
-      source = 'CRAN',
-      version = ver
-    ), class = c('packageRecord', 'CRAN')))
-  } else if (length(df$Repository)) {
-    # It's a package from a custom CRAN-like repo!
-    return(structure(list(
-      name = name,
-      source = as.character(df$Repository),
-      version = ver
-    ), class = c('packageRecord', 'CustomCRANLikeRepository')))
-  } else if (length(df$biocViews)) {
+  } else if (!is.null(df$biocViews)) {
     # It's Bioconductor!
     return(structure(list(
       name = name,
       source = 'Bioconductor',
       version = ver
-    ), class = c('packageRecord', 'Bioconductor')))
-  } else if (name %in% available[, "Package"]) {
-    # It's available on CRAN, so get it from CRAN!
-    return(structure(list(
-      name = name,
-      source = 'CustomCRANLikeRepository',
-      version = ver
-    ), class = c('packageRecord', 'CustomCRANLikeRepository')))
+    ), class=c('packageRecord', 'Bioconductor')))
   } else if (identical(as.character(df$InstallSource), "source")) {
     # It's a local source package!
     return(structure(list(
       name = name,
       source = 'source',
       version = ver
-    ), class = c('packageRecord', 'source')))
+    ), class=c('packageRecord', 'source')))
   } else if ((identical(name, "manipulate") || identical(name, "rstudio")) &&
                identical(as.character(df$Author), "RStudio")) {
     # The 'manipulate' and 'rstudio' packages are auto-installed by RStudio
@@ -417,17 +297,11 @@ inferPackageRecord <- function(df, available = availablePackages()) {
     return(NULL)
   } else {
 
-    # Don't warn if this is an R package being managed by packrat.
-    # NOTE: Not all projects with DESCRIPTION files are R packages!
-    pkgName <- NULL
-    if (isPackratModeOn()) {
-      projectPath <- .packrat_mutables$get("project")
-      if (!is.null(projectPath) && isRPackage(projectPath)) {
-        pkgName <- tryCatch(
-          unname(readDcf(file.path(projectPath, "DESCRIPTION"))[, "Package"]),
-          error = function(e) NULL
-        )
-      }
+    # Don't warn if this is an R package being managed by packrat
+    if (file.exists("DESCRIPTION")) {
+      pkgName <- unname(readDcf("DESCRIPTION")[, "Package"])
+    } else {
+      pkgName <- NULL
     }
 
     if (!identical(pkgName, name)) {
@@ -438,7 +312,7 @@ inferPackageRecord <- function(df, available = availablePackages()) {
       name = name,
       source = 'unknown',
       version = ver
-    ), class = 'packageRecord'))
+    ), class='packageRecord'))
   }
 }
 
@@ -456,7 +330,7 @@ getSourcePackageInfoImpl <- function(path) {
   ## For tarballs, we unzip them to a temporary directory and then read from there
   tempdir <- file.path(tempdir(), "packrat", path)
   if (endswith(path, "tar.gz")) {
-    untar(path, exdir = tempdir, tar = "internal")
+    paths <- untar(path, exdir = tempdir)
     folderName <- list.files(tempdir, full.names = TRUE)[[1]]
   } else {
     folderName <- path
@@ -470,7 +344,7 @@ getSourcePackageInfoImpl <- function(path) {
   data.frame(
     name = as.character(desc$Package),
     version = as.character(desc$Version),
-    path = normalizePath(path, winslash = '/'),
+    path = normalizePath(path, winslash='/'),
     stringsAsFactors = FALSE
   )
 
@@ -490,6 +364,20 @@ pick <- function(property, package, defaultValue = NA) {
   }
 }
 
+# If called without a second argument, returns a curried function. If called
+# with a second argument then it returns the package without the indicated
+# properties.
+strip <- function(properties, package) {
+  func <- function(packageRecord) {
+    packageRecord[!names(packageRecord) %in% properties]
+  }
+  if (!missing(package)) {
+    return(func(package))
+  } else {
+    return(func)
+  }
+}
+
 # Returns a character vector of package names. Depends are ignored.
 pkgNames <- function(packageRecords) {
   if (length(packageRecords) == 0)
@@ -538,7 +426,7 @@ searchPackages <- function(packages, packageNames) {
 # Returns a linear list of package records, sorted by name, with all dependency
 # information removed (or, optionally, reduced to names)
 flattenPackageRecords <- function(packageRecords, depInfo = FALSE, sourcePath = FALSE) {
-  visited <- new.env(parent = emptyenv())
+  visited <- new.env(parent=emptyenv())
   visit <- function(pkgRecs) {
     for (rec in pkgRecs) {
       if (isTRUE(depInfo)) {
@@ -561,26 +449,7 @@ flattenPackageRecords <- function(packageRecords, depInfo = FALSE, sourcePath =
   })
 }
 
-diffableRecord <- function(record) {
-  ignoredFields <- c('depends', 'source_path', 'hash')
-  recordNames <- names(record)
-  recordNames <- setdiff(recordNames, ignoredFields)
-
-  # Remote SHA backwards compatible with cache v2: use 'GithubSHA1' if exists, otherwise all 'Remote' fields
-  if ("gh_sha1" %in% recordNames) {
-    # Remove all the Remote* fields when using GitHub.
-    recordNames <- recordNames[grep("^remote_", recordNames, invert = TRUE)]
-  }
-  record[recordNames]
-}
-
-# debug helper to print a package record. includes field names, type of value, and value.
-printPackageRecord <- function(name, record) {
-  cat(name, "\n")
-  cat(paste(names(record), lapply(record,typeof), record, sep = ":", collapse = "\n"),"\n")
-}
-
-# states: NA (unchanged), remove, add, upgrade, downgrade, crossgrade
+# States: NA (unchanged), remove, add, upgrade, downgrade, crossgrade
 # (crossgrade means name and version was the same but something else was
 # different, i.e. different source or GitHub SHA1 hash or something)
 
@@ -599,20 +468,9 @@ diff <- function(packageRecordsA, packageRecordsB) {
       pkgA <- searchPackages(packageRecordsA, pkgName)[[1]]
       pkgB <- searchPackages(packageRecordsB, pkgName)[[1]]
 
-      strippedA <- diffableRecord(pkgA)
-      strippedB <- diffableRecord(pkgB)
-
-      ## Helpful when debugging unexpected differences between two package records.
-      ##
-      ## printPackageRecord("pkgA", pkgA)
-      ## printPackageRecord("pkgB", pkgB)
-      ## printPackageRecord("strippedA", strippedA)
-      ## printPackageRecord("strippedB", strippedB)
-
-      if (identical(strippedA, strippedB)) {
+      if (identical(strip(c('depends', 'source_path', 'hash'), pkgA),
+                    strip(c('depends', 'source_path', 'hash'), pkgB)))
         return(NA)
-      }
-
       verComp <- compareVersion(pkgA$version, pkgB$version)
       if (verComp < 0)
         return('upgrade')
