@@ -1,26 +1,3 @@
-
-sprintf <- function(fmt, ...) {
-
-  dots <- eval(substitute(alist(...)))
-  if (length(dots) == 0)
-    return(fmt)
-
-  base::sprintf(fmt, ...)
-
-}
-
-stopf <- function(fmt = "", ..., call. = FALSE) {
-  stop(sprintf(fmt, ...), call. = call.)
-}
-
-warningf <- function(fmt = "", ..., call. = FALSE, immediate. = FALSE) {
-  warning(sprintf(fmt, ...), call. = call., immediate. = immediate.)
-}
-
-messagef <- function(fmt = "", ..., appendLF = TRUE) {
-  message(sprintf(fmt, ...), appendLF = appendLF)
-}
-
 silent <- function(expr) {
   suppressWarnings(suppressMessages(
     capture.output(result <- eval(expr, envir = parent.frame()))
@@ -42,7 +19,7 @@ forceUnload <- function(pkg) {
   pkgDLL <- getLoadedDLLs()[[pkgName]]
   if (!is.null(pkgDLL)) {
     suppressWarnings({
-      pkgDir <- system.file(package = pkgName)
+      pkgDir <- system.file(package=pkgName)
       if (nzchar(pkgDir))
         try(library.dynam.unload(pkgName, pkgDir), silent = TRUE)
     })
@@ -69,7 +46,7 @@ list_files <- function(path = ".", pattern = NULL, all.files = FALSE,
 }
 
 # wrapper around read.dcf to workaround LC_CTYPE bug
-# (see: # https://stat.ethz.ch/pipermail/r-devel/2014-May/069046.html)
+# (see: http://r.789695.n4.nabble.com/Bug-in-read-dcf-all-TRUE-td4690578.html)
 readDcf <- function(...) {
   loc <- Sys.getlocale('LC_CTYPE')
   on.exit(Sys.setlocale('LC_CTYPE', loc))
@@ -85,9 +62,6 @@ is_dir <- function(file) {
 dir_copy <- function(from, to, overwrite = FALSE, all.files = TRUE,
                      pattern = NULL, ignore.case = TRUE) {
 
-  owd <- getwd()
-  on.exit(setwd(owd), add = TRUE)
-
   # Make sure we're doing sane things
   if (!is_dir(from)) stop("'", from, "' is not a directory.")
 
@@ -95,7 +69,7 @@ dir_copy <- function(from, to, overwrite = FALSE, all.files = TRUE,
     if (overwrite) {
       unlink(to, recursive = TRUE)
     } else {
-      stop(paste(sep = "",
+      stop(paste( sep = "",
                   if (is_dir(to)) "Directory" else "File",
                   " already exists at path '", to, "'."
       ))
@@ -151,11 +125,6 @@ pkgDescriptionDependencies <- function(file) {
 
   if (!file.exists(file)) stop("no file '", file, "'")
   DESCRIPTION <- readDcf(file)
-
-  # ignore empty description
-  if (nrow(DESCRIPTION) < 1)
-    return(list())
-
   requirements <- DESCRIPTION[1, fields[fields %in% colnames(DESCRIPTION)]]
 
   ## Remove whitespace
@@ -194,11 +163,11 @@ pkgDescriptionDependencies <- function(file) {
 
   ## Don't include 'base' packages
   ip <- installed.packages()
-  basePkgs <- ip[Vectorize(isTRUE)(ip[, "Priority"] == "base"), "Package"]
-  result <- result[!(result$Package %in% basePkgs), ]
+  basePkgs <- ip[ Vectorize(isTRUE)(ip[, "Priority"] == "base"), "Package" ]
+  result <- result[ !(result$Package %in% basePkgs), ]
 
   ## Don't include R
-  result <- result[!result$Package == "R", ]
+  result <- result[ !result$Package == "R", ]
 
   result
 
@@ -217,7 +186,7 @@ startswith <- function(str1, str2) {
 
 # does str1 end with str2?
 endswith <- function(str1, str2) {
-  if (!length(str2) == 1) stop("expecting a length 1 string for 'str2'")
+  if (!length(str2) == 1) stop ("expecting a length 1 string for 'str2'")
   n2 <- nchar(str2)
   sapply(str1, function(x) {
     nx <- nchar(x)
@@ -228,21 +197,10 @@ endswith <- function(str1, str2) {
   })
 }
 
-stopIfNoLockfile <- function(project) {
-  path <- lockFilePath(project)
-  if (!file.exists(path)) {
-    stop("This project does not have a lockfile. (Have you called 'packrat::snapshot()' yet?)",
-         call. = FALSE)
-  }
-
-  invisible(TRUE)
-}
-
 stopIfNotPackified <- function(project) {
-
   if (!checkPackified(project, quiet = TRUE)) {
     if (identical(project, getwd())) {
-      stop("This project has not yet been packified.\nRun 'packrat::init()' to init packrat.",
+      stop("This project has not yet been packified.\nRun 'packrat::init() to init packrat.",
            call. = FALSE)
     } else {
       stop("The project at '", project, "' has not yet been packified.\nRun 'packrat::init('", project, "') to init packrat.",
@@ -251,41 +209,30 @@ stopIfNotPackified <- function(project) {
   }
 }
 
-# Expected to be used with .Rbuildignore, .Rinstignore
-# .gitignore + SVN ignore have their own (similar) logical, but
-# need to handle options specially
+## Expected to be used with .Rbuildignore, .Rinstignore
+## .gitignore + SVN ignore have their own (similar) logical, but
+## need to handle options specially
 updateIgnoreFile <- function(project = NULL, file, add = NULL, remove = NULL) {
 
   project <- getProjectDir(project)
 
-  # if no ignore file exists, populate it
+  ## If the file doesn't exist, create and fill it
   path <- file.path(project, file)
   if (!file.exists(path)) {
-    if (length(add) > 0)
-      cat(add, file = path, sep = "\n")
+    cat(add, file = path, sep = "\n")
     return(invisible())
   }
 
-  # read ignore file and track changes
-  oldContent <- readLines(path)
-  newContent <- oldContent
-
-  # add items to end of ignore file (avoid duplication of entries)
-  if (length(add))
-    newContent <- c(newContent, setdiff(add, newContent))
-
-  # remove items from ignore file
-  if (length(remove))
-    newContent <- newContent[!(newContent %in% remove)]
-
-  # only mutate ignore file if contents have indeed changed
-  if (!identical(oldContent, newContent))
-    cat(newContent, file = path, sep = "\n")
-
+  ## If it already exists, add and remove as necessary
+  content <- readLines(path)
+  content <- union(content, add)
+  content <- setdiff(content, remove)
+  cat(content, file = path, sep = "\n")
   return(invisible())
+
 }
 
-updateRBuildIgnore <- function(project = NULL) {
+updateRBuildIgnore <- function(project = NULL, options) {
 
   add <- c(
     "^packrat/",
@@ -315,17 +262,9 @@ updateGitIgnore <- function(project = NULL, options) {
                    remove = remove)
 }
 
-# A packrat project is managed by git if any one of its parent directories
-# contains a '.git' folder.
 isGitProject <- function(project) {
-  path <- project
-  while (dirname(path) != path) {
-    .git <- file.path(path, ".git")
-    if (file.exists(.git) && is_dir(.git))
-      return(TRUE)
-    path <- dirname(path)
-  }
-  return(FALSE)
+  .git <- file.path(project, ".git")
+  file.exists(.git) && is_dir(.git)
 }
 
 isSvnProject <- function(project) {
@@ -392,27 +331,10 @@ setLibPaths <- function(paths) {
   .libPaths(paths)
 }
 
-normalize_paths <- function(paths, winslash = "/", mustWork = FALSE) {
-  paths[paths == ""] <- getwd()
-  unlist(lapply(paths, function(path) {
-    normalizePath(path, winslash = "/", mustWork = FALSE)
-  }))
-}
-
-getLibPaths <- function() {
-  normalize_paths(.libPaths())
-}
-
-getUserLibPaths <- function() {
-  allPaths <- getLibPaths()
-  sysPaths <- normalize_paths(c(.Library, .Library.site))
-  setdiff(allPaths, sysPaths)
-}
-
-## Get the default library paths (those that would be used upon
-## starting a new R session)
-getDefaultLibPaths <- function() {
-  getenv("R_PACKRAT_DEFAULT_LIBPATHS")
+## We only want to grab user libraries here -- system libraries are automatically
+## added in by R
+getLibPaths <- function(paths) {
+  setdiff(.libPaths(), c(.Library, .Library.site))
 }
 
 getInstalledPkgInfo <- function(packages, installed.packages, ...) {
@@ -507,9 +429,18 @@ loadedNamespacePaths <- function() {
   result
 }
 
+# Work around namespace:stats potentially not being loaded
+setNames <- function(object = nm, nm) {
+  names(object) <- nm
+  object
+}
+
 # Drop null values in a list
+# Note that we use unlist(lapply(...)) because, if 'x'
+# is an empty list, then 'unlist(list())' will return NULL,
+# and list()[NULL] returns list() as we would hope
 dropNull <- function(x) {
-  Filter(Negate(is.null), x)
+  x[unlist(lapply(x, Negate(is.null)))]
 }
 
 surround <- function(x, with = "'") {
@@ -524,19 +455,8 @@ write_dcf <- function(x, file = "", append = FALSE, indent = 4,
 }
 
 symlink <- function(from, to) {
-
-  # attempt to generating the symlink
-  if (is.windows())
-    Sys.junction(from, to)
-  else
-    file.symlink(from, to)
-
-  # check to see if the file was properly generated
-  file.exists(to)
-}
-
-nullfile <- function() {
-  if (is.windows()) "NUL" else "/dev/null"
+  if (is.windows()) Sys.junction(from, to)
+  else file.symlink(from, to)
 }
 
 with_dir <- function(dir, expr) {
@@ -560,166 +480,3 @@ with_collate <- function(locale, code) {
 }
 
 sort_c <- function(x) with_collate("C", sort(x))
-
-is.string <- function(x) {
-  is.character(x) && length(x) == 1
-}
-
-is.directory <- function(x) {
-  file.exists(x) && isTRUE(file.info(x)[["isdir"]]) # guard against NA
-}
-
-getBinaryPkgType <- function() {
-  .Platform$pkgType
-}
-
-normalize.path <- function(path) {
-  normalizePath(path, winslash = "/", mustWork = TRUE)
-}
-
-filePrefix <- function() {
-  if (is.windows())
-    "file:///"
-  else
-    "file://"
-}
-
-reFilePrefix <- function() {
-  paste("^", filePrefix(), sep = "")
-}
-
-isProgramOnPath <- function(program) {
-  nzchar(Sys.which(program)[[1]])
-}
-
-isPathToSameFile <- function(lhs, rhs) {
-
-  if (!(is.string(lhs) && is.string(rhs)))
-    return(FALSE)
-
-  lhsNorm <- normalizePath(lhs, winslash = "/", mustWork = FALSE)
-  rhsNorm <- normalizePath(rhs, winslash = "/", mustWork = FALSE)
-
-  lhsNorm == rhsNorm
-
-}
-
-isTestingPackrat <- function() {
-  !is.na(Sys.getenv("R_PACKRAT_TESTING", unset = NA))
-}
-
-defer <- function(expr, envir = parent.frame()) {
-
-  # Create a call that must be evaluated in the parent frame (as
-  # that's where functions and symbols need to be resolved)
-  call <- substitute(
-    evalq(expr, envir = envir),
-    list(expr = substitute(expr), envir = parent.frame())
-  )
-
-  # Use 'do.call' with 'on.exit' to attach the evaluation to
-  # the exit handlrs of the selected frame
-  do.call(base::on.exit, list(substitute(call), add = TRUE), envir = envir)
-}
-
-join <- function(..., sep = "", collapse = NULL) {
-  paste(..., sep = sep, collapse = collapse)
-}
-
-# sneakily get a function
-yoink <- function(package, symbol) {
-  eval(call(":::", package, symbol))
-}
-
-enumerate <- function(list, fn) {
-  keys <- names(list)
-  values <- list
-  sapply(seq_along(keys), function(i) {
-    fn(keys[[i]], values[[i]])
-  })
-}
-
-packageVersionInstalled <- function(...) {
-  enumerate(list(...), function(package, version) {
-    result <- try(packageVersion(package), silent = TRUE)
-    !inherits(result, "try-error") && result >= version
-  })
-}
-
-canUseHttr <- function() {
-  packageVersionInstalled(httr = "1.0.0")
-}
-
-packratOption <- function(envName, optionName, defaultValue) {
-
-  envValue <- Sys.getenv(envName, unset = NA)
-  if (!is.na(envValue))
-    return(envValue)
-
-  optionValue <- getOption(optionName)
-  if (!is.null(optionValue))
-    return(optionValue)
-
-  defaultValue
-}
-
-packratOptionBoolean <- function(envName, optionName, defaultValue) {
-
-  option <- packratOption(envName, optionName, defaultValue)
-
-  if (is.character(option)) {
-    option <- if (!nzchar(option))
-      FALSE
-    else if (tolower(option) %in% c("t", "true", "y", "yes"))
-      TRUE
-    else if (tolower(option) %in% c("f", "false", "n", "no"))
-      FALSE
-    else
-      as.logical(eval(parse(text = option)))
-  }
-
-  as.logical(option)
-}
-
-ensureDirectory <- function(path) {
-
-  info <- file.info(path)
-  if (identical(info$isdir, TRUE))
-    return(path)
-  else if (identical(info$isdir, FALSE))
-    stop("path '", path, "' exists but is not a directory")
-  else if (!dir.create(path, recursive = TRUE))
-    stop("failed to create directory at path '", path, "'")
-
-  path
-}
-
-quietly <- function(expr) {
-  withCallingHandlers(
-    tryCatch(expr = expr, error = identity),
-    warning = function(w) invokeRestart("muffleWarning"),
-    message = function(m) invokeRestart("muffleMessage")
-  )
-}
-
-onError <- function(default, expr) {
-  tryCatch(expr, error = function(e) default)
-}
-
-# logger
-logTimestamper <- function() {
-  paste("[", as.character(Sys.time()), " packrat]", sep = "")
-}
-
-timestampedLog <- function(...) {
-  cat(paste(logTimestamper(), ..., "\n"))
-}
-
-# Returns a logging function when enabled, a noop function otherwise.
-verboseLogger <- function(verbose) {
-  if (verbose) {
-    timestampedLog
-  } else {
-    function(...) {}
-  }
-}
