@@ -1,39 +1,9 @@
 local({
 
-  ## Helper function to get the path to the library directory for a
-  ## given packrat project.
-  getPackratLibDir <- function(projDir = NULL) {
-    path <- file.path("packrat", "lib", R.version$platform, getRversion())
+  libDir <- file.path('packrat', 'lib', R.version$platform, getRversion())
 
-    if (!is.null(projDir)) {
-
-      ## Strip trailing slashes if necessary
-      projDir <- sub("/+$", "", projDir)
-
-      ## Only prepend path if different from current working dir
-      if (!identical(normalizePath(projDir), normalizePath(getwd())))
-        path <- file.path(projDir, path)
-    }
-
-    path
-  }
-
-  ## Ensure that we set the packrat library directory relative to the
-  ## project directory. Normally, this should be the working directory,
-  ## but we also use '.rs.getProjectDirectory()' if necessary (e.g. we're
-  ## rebuilding a project while within a separate directory)
-  libDir <- if (exists(".rs.getProjectDirectory"))
-    getPackratLibDir(.rs.getProjectDirectory())
-  else
-    getPackratLibDir()
-
-  ## Unload packrat in case it's loaded -- this ensures packrat _must_ be
-  ## loaded from the private library. Note that `requireNamespace` will
-  ## succeed if the package is already loaded, regardless of lib.loc!
-  if ("packrat" %in% loadedNamespaces())
-    try(unloadNamespace("packrat"), silent = TRUE)
-
-  if (suppressWarnings(requireNamespace("packrat", quietly = TRUE, lib.loc = libDir))) {
+  if (is.na(Sys.getenv("RSTUDIO_PACKRAT_BOOTSTRAP", unset = NA)) &&
+        suppressWarnings(requireNamespace("packrat", quietly = TRUE, lib.loc = libDir))) {
 
     # Check 'print.banner.on.startup' -- when NA and RStudio, don't print
     print.banner <- packrat::get_opts("print.banner.on.startup")
@@ -45,30 +15,18 @@ local({
     return(packrat::on(print.banner = print.banner))
   }
 
-  ## Escape hatch to allow RStudio to handle bootstrapping. This
-  ## enables RStudio to provide print output when automagically
-  ## restoring a project from a bundle on load.
-  if (!is.na(Sys.getenv("RSTUDIO", unset = NA)) &&
-      is.na(Sys.getenv("RSTUDIO_PACKRAT_BOOTSTRAP", unset = NA))) {
-    Sys.setenv("RSTUDIO_PACKRAT_BOOTSTRAP" = "1")
-    setHook("rstudio.sessionInit", function(...) {
-      # Ensure that, on sourcing 'packrat/init.R', we are
-      # within the project root directory
-      if (exists(".rs.getProjectDirectory")) {
-        owd <- getwd()
-        setwd(.rs.getProjectDirectory())
-        on.exit(setwd(owd), add = TRUE)
-      }
-      source("packrat/init.R")
-    })
-    return(invisible(NULL))
-  }
+  ## Bootstrapping -- only performed in interactive contexts
+  if (interactive()) {
 
-  ## Bootstrapping -- only performed in interactive contexts,
-  ## or when explicitly asked for on the command line
-  if (interactive() || "--bootstrap-packrat" %in% commandArgs(TRUE)) {
-
-    needsRestore <- "--bootstrap-packrat" %in% commandArgs(TRUE)
+    ## Escape hatch to allow RStudio to handle initialization
+    if (!is.na(Sys.getenv("RSTUDIO", unset = NA)) &&
+          is.na(Sys.getenv("RSTUDIO_PACKRAT_BOOTSTRAP", unset = NA))) {
+      Sys.setenv("RSTUDIO_PACKRAT_BOOTSTRAP" = "1")
+      setHook("rstudio.sessionInit", function(...) {
+        source("packrat/init.R")
+      })
+      return(invisible(NULL))
+    }
 
     message("Packrat is not installed in the local library -- ",
             "attempting to bootstrap an installation...")
@@ -106,7 +64,7 @@ local({
       }
 
       ## Try downloading packrat from CRAN if available
-      else if ("packrat" %in% available.packages()[, "Package"]) {
+      else if ("packrat" %in% rownames(available.packages())) {
         message("> Installing packrat from CRAN")
         install.packages("packrat")
       }
@@ -119,8 +77,7 @@ local({
       }
 
       # Restore the project, unload the temporary packrat, and load the private packrat
-      if (needsRestore)
-        packrat::restore(prompt = FALSE, restart = TRUE)
+      packrat::restore(prompt = FALSE, restart = TRUE)
 
       ## This code path only reached if we didn't restart earlier
       unloadNamespace("packrat")
@@ -143,6 +100,7 @@ local({
     if (!file.exists(lib)) {
       dir.create(lib, recursive = TRUE)
     }
+    lib <- normalizePath(lib, winslash = "/")
 
     message("> Installing packrat into project private library:")
     message("- ", shQuote(lib))
@@ -152,8 +110,7 @@ local({
       paste0(with, x, with)
     }
 
-
-    ## Invoke install.packages() in clean R session
+    ## The following is performed because a regular install.packages call can fail
     peq <- function(x, y) paste(x, y, sep = " = ")
     installArgs <- c(
       peq("pkgs", surround(packratSrcPath, with = "'")),
@@ -161,22 +118,17 @@ local({
       peq("repos", "NULL"),
       peq("type", surround("source", with = "'"))
     )
-
-    fmt <- "utils::install.packages(%s)"
-    installCmd <- sprintf(fmt, paste(installArgs, collapse = ", "))
-
-    ## Write script to file (avoid issues with command line quoting
-    ## on R 3.4.3)
-    installFile <- tempfile("packrat-bootstrap", fileext = ".R")
-    writeLines(installCmd, con = installFile)
-    on.exit(unlink(installFile), add = TRUE)
+    installCmd <- paste(sep = "",
+                        "utils::install.packages(",
+                        paste(installArgs, collapse = ", "),
+                        ")")
 
     fullCmd <- paste(
       surround(file.path(R.home("bin"), "R"), with = "\""),
       "--vanilla",
-      "-s",
-      "-f",
-      surround(installFile, with = "\"")
+      "--slave",
+      "-e",
+      surround(installCmd, with = "\"")
     )
     system(fullCmd)
 
@@ -186,10 +138,10 @@ local({
     ## an 'installed from source' version
 
     ## -- InstallAgent -- ##
-    installAgent <- "InstallAgent: packrat 0.8.1"
+    installAgent <- 'InstallAgent: packrat 0.4.1.8'
 
     ## -- InstallSource -- ##
-    installSource <- "InstallSource: source"
+    installSource <- 'InstallSource: source'
 
     packratDescPath <- file.path(lib, "packrat", "DESCRIPTION")
     DESCRIPTION <- readLines(packratDescPath)
@@ -201,8 +153,7 @@ local({
     library("packrat", character.only = TRUE, lib.loc = lib)
 
     message("> Restoring library")
-    if (needsRestore)
-      packrat::restore(prompt = FALSE, restart = FALSE)
+    restore(restart = FALSE)
 
     # If the environment allows us to restart, do so with a call to restore
     restart <- getOption("restart")
@@ -214,7 +165,7 @@ local({
 
     # Callers (source-erers) can define this hidden variable to make sure we don't enter packrat mode
     # Primarily useful for testing
-    if (!exists(".__DONT_ENTER_PACKRAT_MODE__.") && interactive()) {
+    if (!exists(".__DONT_ENTER_PACKRAT_MODE__.")) {
       message("> Packrat bootstrap successfully completed. Entering packrat mode...")
       packrat::on()
     }
